@@ -1,6 +1,6 @@
 // * This makes emacs happy -*-Mode: C++;-*-
 /****************************************************************************
- * Copyright (c) 1998-2007,2008 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2012,2018 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -31,7 +31,7 @@
  *   Author: Juergen Pfeifer, 1997                                          *
  ****************************************************************************/
 
-// $Id: internal.h,v 1.17 2008/12/07 02:07:34 juergen Exp $
+// $Id: internal.h,v 1.19 2018/06/24 00:10:14 tom Exp $
 
 #ifndef NCURSES_CPLUS_INTERNAL_H
 #define NCURSES_CPLUS_INTERNAL_H 1
@@ -41,10 +41,10 @@
 #if USE_RCS_IDS
 #define MODULE_ID(id) static const char Ident[] = id;
 #else
-#define MODULE_ID(id) /*nothing*/
+#define MODULE_ID(id)		/*nothing */
 #endif
 
-#ifdef __MINGW32__
+#ifdef _WIN32
 #include <nc_mingw.h>
 #undef KEY_EVENT
 #endif
@@ -62,4 +62,6 @@
 #define NULL 0
 #endif
 
+#include <nc_string.h>
+
 #endif /* NCURSES_CPLUS_INTERNAL_H */
