@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2004,2010 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,11 +28,11 @@
 
 /****************************************************************************
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
- *   Contact: http://www.familiepfeifer.de/Contact.aspx?Lang=en             *
  ****************************************************************************/
+
 #include "form.priv.h"
 
-MODULE_ID("$Id: fld_opts.c,v 1.8 2002/07/06 15:33:27 juergen Exp $")
+MODULE_ID("$Id: fld_opts.c,v 1.12 2010/01/23 21:14:36 tom Exp $")
 
 /*----------------------------------------------------------------------------
   Field-Options manipulation routines
@@ -51,12 +51,15 @@ MODULE_ID("$Id: fld_opts.c,v 1.8 2002/07/06 15:33:27 juergen Exp $")
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_field_opts (FIELD * field, Field_Options opts)
+set_field_opts(FIELD *field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
+
+  T((T_CALLED("set_field_opts(%p,%d)"), (void *)field, opts));
+
   opts &= ALL_FIELD_OPTS;
   if (!(opts & ~ALL_FIELD_OPTS))
-    res = _nc_Synchronize_Options( Normalize_Field(field), opts );
+    res = _nc_Synchronize_Options(Normalize_Field(field), opts);
   RETURN(res);
 }
 
@@ -69,9 +72,11 @@ set_field_opts (FIELD * field, Field_Options opts)
 |   Return Values :  The options.
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(Field_Options)
-field_opts (const FIELD * field)
+field_opts(const FIELD *field)
 {
-  return ALL_FIELD_OPTS & Normalize_Field( field )->opts;
+  T((T_CALLED("field_opts(%p)"), (const void *)field));
+
+  returnCode(ALL_FIELD_OPTS & Normalize_Field(field)->opts);
 }
 
 /*---------------------------------------------------------------------------
@@ -87,15 +92,17 @@ field_opts (const FIELD * field)
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-field_opts_on (FIELD * field, Field_Options opts)
+field_opts_on(FIELD *field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
 
+  T((T_CALLED("field_opts_on(%p,%d)"), (void *)field, opts));
+
   opts &= ALL_FIELD_OPTS;
   if (!(opts & ~ALL_FIELD_OPTS))
     {
-      Normalize_Field( field );
-      res = _nc_Synchronize_Options( field, field->opts | opts );
+      Normalize_Field(field);
+      res = _nc_Synchronize_Options(field, field->opts | opts);
     }
   RETURN(res);
 }
@@ -113,17 +120,19 @@ field_opts_on (FIELD * field, Field_Options opts)
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-field_opts_off (FIELD  * field, Field_Options opts)
+field_opts_off(FIELD *field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
 
+  T((T_CALLED("field_opts_off(%p,%d)"), (void *)field, opts));
+
   opts &= ALL_FIELD_OPTS;
   if (!(opts & ~ALL_FIELD_OPTS))
     {
-      Normalize_Field( field );
-      res = _nc_Synchronize_Options( field, field->opts & ~opts );
+      Normalize_Field(field);
+      res = _nc_Synchronize_Options(field, field->opts & ~opts);
     }
   RETURN(res);
-}	
+}
 
 /* fld_opts.c ends here */
