@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2010,2012 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,12 +28,11 @@
 
 /****************************************************************************
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
- *   Contact: http://www.familiepfeifer.de/Contact.aspx?Lang=en             *
  ****************************************************************************/
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: frm_page.c,v 1.7 2002/07/06 15:33:27 juergen Exp $")
+MODULE_ID("$Id: frm_page.c,v 1.12 2012/06/10 00:28:04 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -48,35 +47,37 @@ MODULE_ID("$Id: frm_page.c,v 1.7 2002/07/06 15:33:27 juergen Exp $")
 |                    E_SYSTEM_ERROR    - system error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_form_page (FORM * form, int page)
+set_form_page(FORM *form, int page)
 {
   int err = E_OK;
 
-  if ( !form || (page<0) || (page>=form->maxpage) )
+  T((T_CALLED("set_form_page(%p,%d)"), (void *)form, page));
+
+  if (!form || (page < 0) || (page >= form->maxpage))
     RETURN(E_BAD_ARGUMENT);
 
   if (!(form->status & _POSTED))
     {
-      form->curpage = page;
+      form->curpage = (short)page;
       form->current = _nc_First_Active_Field(form);
-  }
+    }
   else
     {
-      if (form->status & _IN_DRIVER) 
+      if (form->status & _IN_DRIVER)
 	err = E_BAD_STATE;
       else
 	{
 	  if (form->curpage != page)
 	    {
-	      if (!_nc_Internal_Validation(form)) 
+	      if (!_nc_Internal_Validation(form))
 		err = E_INVALID_FIELD;
 	      else
 		{
-		  Call_Hook(form,fieldterm);
-		  Call_Hook(form,formterm);
-		  err = _nc_Set_Form_Page(form,page,(FIELD *)0);
-		  Call_Hook(form,forminit);
-		  Call_Hook(form,fieldinit);
+		  Call_Hook(form, fieldterm);
+		  Call_Hook(form, formterm);
+		  err = _nc_Set_Form_Page(form, page, (FIELD *)0);
+		  Call_Hook(form, forminit);
+		  Call_Hook(form, fieldinit);
 		  _nc_Refresh_Current_Field(form);
 		}
 	    }
@@ -95,9 +96,11 @@ set_form_page (FORM * form, int page)
 |                    -1    : invalid form pointer
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-form_page (const FORM * form)
+form_page(const FORM *form)
 {
-  return Normalize_Form(form)->curpage;
+  T((T_CALLED("form_page(%p)"), (const void *)form));
+
+  returnCode(Normalize_Form(form)->curpage);
 }
 
 /* frm_page.c ends here */
