@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2008-2010,2012 Free Software Foundation, Inc.              *
+ * Copyright (c) 2008-2012,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -34,7 +34,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: fty_generic.c,v 1.6 2012/06/10 00:27:49 tom Exp $")
+MODULE_ID("$Id: fty_generic.c,v 1.7 2016/01/23 21:32:00 tom Exp $")
 
 /*
  * This is not a full implementation of a field type, but adds some
@@ -110,7 +110,11 @@ _nc_generic_fieldtype(bool (*const field_check) (FORM *, FIELD *, const void *),
   FIELDTYPE *res = (FIELDTYPE *)0;
 
   T((T_CALLED("_nc_generic_fieldtype(%p,%p,%p,%p,%p)"),
-     field_check, char_check, next, prev, freecallback));
+     TR_FUNC(field_check),
+     TR_FUNC(char_check),
+     TR_FUNC(next),
+     TR_FUNC(prev),
+     TR_FUNC(freecallback)));
 
   if (field_check || char_check)
     {
