@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998-2000,2005 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
  ****************************************************************************/
 
 /*
- * $Id: SigAction.h,v 1.5 1999/06/19 23:00:54 tom Exp $
+ * $Id: SigAction.h,v 1.9 2005/08/06 20:05:32 tom Exp $
  *
  * This file exists to handle non-POSIX systems which don't have <unistd.h>,
  * and usually no sigaction() nor <termios.h>
@@ -59,10 +59,6 @@ typedef struct sigaction sigaction_t;
 
 #if HAVE_SIGVEC
 
-#if HAVE_LIBC_H
-#include <libc.h>
-#endif
-
 #undef  SIG_BLOCK
 #define SIG_BLOCK       00
 
@@ -105,12 +101,8 @@ typedef unsigned long sigset_t;
 #undef  sigaddset
 #define sigaddset   _nc_sigaddset
 
-extern int sigaction (int sig, sigaction_t * sigact, sigaction_t *  osigact);
-extern int sigprocmask (int how, sigset_t *mask, sigset_t *omask);
-extern int sigemptyset (sigset_t *mask);
-extern int sigsuspend (sigset_t *mask);
-extern int sigdelset (sigset_t *mask, int sig);
-extern int sigaddset (sigset_t *mask, int sig);
+/* tty/lib_tstp.c is the only user */
+#include <base/sigaction.c>
 
 #endif /* HAVE_SIGVEC */
 #endif /* HAVE_SIGACTION */
